#include <glib.h>
#include "gbox_menu.h"

GboxMenuItem *
gbox_menu_item_new(gchar *name)
{
  GboxMenuItem *item;
  
  item = (GboxMenuItem *) malloc(sizeof(GboxMenuItem));
  
  item->nickname = g_strdup(name);
  item->item_count = 0;
  item->next = NULL;
  
  return item;
}

GboxMenu *
gbox_menu_new(AppletWidget *app)
{
  GboxMenu *menu;
  
  menu = (GboxMenu *)malloc(sizeof(GboxMenu));
  
  menu->menu_count = 0;
  menu->menu_first = NULL;
  menu->menu_last = NULL;
  menu->applet = app;
  
  return menu;
}
  
void 
gbox_menu_free(GboxMenu *menu){
  while(menu->menu_first) 
    gbox_menu_remove_item(menu, menu->menu_first);

  g_free(menu);
  menu = NULL;
}

void
gbox_menu_add_box(GboxMenu *menu, MailBox *box)
{
  GboxMenuItem *new_item = gbox_menu_item_new(box->nickname);
  MailEntry    *mail_walk, *mails;
  int          i;
  gchar        *menu_item = NULL, *menu_name = NULL;
  
  if (menu->menu_last) menu->menu_last->next = new_item;
  menu->menu_last = new_item;
  if (!menu->menu_first) menu->menu_first = new_item;
  
  applet_widget_register_callback_dir(APPLET_WIDGET(menu->applet),
				      box->nickname, box->nickname);
  new_item->item_count = gbox_mailbox_count_mails(box->mailfile, &mails, TRUE);
  
  mail_walk = mails;
  for (i = 0; i < new_item->item_count; i++) {
    switch(box->menu_type) {
    case MENU_TYPE_SUBJECT:
      if (mail_walk->subject)
	menu_item = g_strdup_printf("%s", mail_walk->subject);
      else
	menu_item = g_strdup_printf("%s", _("no subject"));
      break;
    case MENU_TYPE_FROM:
      if (mail_walk->from)
	menu_item = g_strdup_printf("%s", mail_walk->from);
      else
	menu_item = g_strdup_printf("%s", _("no sender"));
      break;
    case MENU_TYPE_BOTH:
      if (mail_walk->from && mail_walk->subject)
	menu_item = g_strdup_printf("%s -- %s", mail_walk->from, mail_walk->subject);
      else if (mail_walk->from)
	menu_item = g_strdup_printf("%s -- %s", mail_walk->from, _("no subject"));
      else 
	menu_item = g_strdup_printf("%s -- %s", _("no sender"), mail_walk->subject);
    }
    
    menu_name = g_strdup_printf("%s/mail%d", box->nickname, i);
    
/*      printf("name: %s item: %s\n", menu_name, menu_item); */
    
    applet_widget_register_callback(APPLET_WIDGET(menu->applet),
				    menu_name, 
				    menu_item, gbox_menu_dummy_callback, NULL);
    g_free(menu_item);
    g_free(menu_name);
    mail_walk = mail_walk->next;
  }
  
  gbox_mailentries_free(&mails);
}
  
void
gbox_menu_remove_item(GboxMenu *menu, GboxMenuItem *item)
{
  int          i;
  gchar        *menu_name;
  GboxMenuItem *walk = menu->menu_first, *prev = NULL;
  
  for (i = 0; i < item->item_count; i++) {
    menu_name = g_strdup_printf("%s/mail%d", item->nickname, i);
    applet_widget_unregister_callback(APPLET_WIDGET(menu->applet),
				      menu_name);
    g_free(menu_name);
  }
  applet_widget_unregister_callback_dir(APPLET_WIDGET(menu->applet),
					item->nickname);
  
  while (walk) {
    if (!strcmp(walk->nickname, item->nickname)) break;
    
    prev = walk;
    walk = walk->next;
  }
  
  if (!walk) return;
  
  if (!prev) {   /* remove first; */
    menu->menu_first = menu->menu_first->next;
  } else {
    prev->next = walk->next;
  }
  
  if (menu->menu_last == walk) menu->menu_last = prev;

  g_free(walk->nickname);
  g_free(walk);
}

void
gbox_menu_remove_box(GboxMenu *menu, MailBox *box)
{
  GboxMenuItem *item = gbox_menu_get_item(menu, box->nickname);
  int          i;
  gchar        *menu_item;
  GboxMenuItem *walk = menu->menu_first, *prev = NULL;
  
  if (!item) return;
  
  for (i = 0; i < item->item_count; i++) {
    menu_item = g_strdup_printf("%s/mail%d", box->nickname, i);
    applet_widget_unregister_callback(APPLET_WIDGET(menu->applet),
				      menu_item);
    g_free(menu_item);
  }
  applet_widget_unregister_callback_dir(APPLET_WIDGET(menu->applet),
					box->nickname);
  
  while (walk) {
    if (!strcmp(walk->nickname, box->nickname)) break;
    
    prev = walk;
    walk = walk->next;
  }
  
  if (!walk) return;
  
  if (!prev) {   /* remove first; */
    menu->menu_first = menu->menu_first->next;
  } else {
    prev->next = walk->next;
  }
  
  if (menu->menu_last == walk) menu->menu_last = prev;

  g_free(walk->nickname);
  g_free(walk);
}

GboxMenuItem *
gbox_menu_get_item(GboxMenu *menu, gchar *name)
{
  GboxMenuItem *walk = menu->menu_first;
  
  while (walk) {
    if (!strcmp(walk->nickname, name)) break;
    walk = walk->next;
  }
  
  return walk;
}

  
void
gbox_menu_dummy_callback(AppletWidget *app, gpointer data)
{
}

